Reservation

Struct Reservation 

pub struct Reservation<'a> { /* private fields */ }
A wrapper around a slot of memory in the ring buffer.

A Reservation has a predetermined size that is determined by the size passed into reserve. Any attempt to write more than this amount of data into the slot is a programming error and will cause an assertion failure. This class provides a formal way for writers to serialize data in place in the ring buffer, thus eliminating the need for a temporary serialization buffer.

Implementations

Source

impl<'a> Reservation<'a>

pub fn write(&mut self, data: &[u8]) -> Result<(), Status>

Writes the given data into this reservation.

pub fn commit(self) -> Result<(), Status>

Advances the write pointer of the associated spsc buffer.

This makes the written data visible to the reader, and thus can only be called once all writes have been completed and the reservation is fully written.